from StatisticReader import StatisticReader

class SolvingTimeReader(StatisticReader):
   '''
   extracts the solving time
   '''
   name = 'SolvingTimeReader'
   regular_exp = 'Solving Time (sec) :'
   columnwidth = 12
   datakey = 'SolvingTime'
   columnheaderstr = 'Time(s)'.rjust(columnwidth)
    
   solvingtimereadkeys = {
      StatisticReader.SOLVERTYPE_SCIP : "Solving Time (sec) :",
      StatisticReader.SOLVERTYPE_CPLEX : "Solution time =",
      StatisticReader.SOLVERTYPE_GUROBI : "Explored ",
      StatisticReader.SOLVERTYPE_CBC : "Coin:Total time (CPU seconds):",
      StatisticReader.SOLVERTYPE_XPRESS : " *** Search "
   }
                   
   solvingtimelineindex = {
      StatisticReader.SOLVERTYPE_SCIP : -1,
      StatisticReader.SOLVERTYPE_CPLEX : 3,
      StatisticReader.SOLVERTYPE_GUROBI : -2,
      StatisticReader.SOLVERTYPE_CBC : 4,
      StatisticReader.SOLVERTYPE_XPRESS : 5
   }
    
   returned = False
   DEFAULT_SOLVINGTIME=3600.0

   def extractStatistic(self, line):
      if self.solvingtimereadkeys[StatisticReader.solvertype] in line:
         solvingtime = line.split()[self.solvingtimelineindex[StatisticReader.solvertype]]
         self.returned = True
         return float(solvingtime)
      else:
         return None

   def execEndOfProb(self):
      if not self.returned:
         return SolvingTimeReader.DEFAULT_SOLVINGTIME
      else:
         self.returned = False
         return None


from DataCollector import DataCollector
from StatisticReader import StatisticReader
from DataCollector_SoluFileDataCollector import SoluFileDataCollector
from StatisticReader_TimeLimitReader import TimeLimitReader
from StatisticReader_PrimalBoundReader import PrimalBoundReader
from TestRun import TestRun
import Misc

class Comparator:
   '''
   manages the collection of all log (.out) and .solu file data
   '''
   files = []
   testruns = []
   datacollector = None
   solufiledatacollector = None
   readers = []
   probnamelist = []
   datakeys = []

   datakey_err = 'ERROR: Datakey is not a suitable key'
   probname_err = 'ERROR: Probname is not in list of problem names'

   datakey_gap = 'SoluFileGap'

   EXCLUDE_REASON_TIMEOUT = 'timout'
   EXCLUDE_REASON_INFEASIBLE = 'infeasible'
   EXCLUDE_REASON_ZEROSOLUTION = 'zerosolution'
   EXCLUDE_REASON_NOSOLUTIONKNOWN = 'nosolution'
   EXCLUDE_REASON_NOOPTIMALSOLUTIONKNOWN = 'nooptsolution'
   INFINITY = 1e09
   COMP_SIGN_LE = 1
   COMP_SIGN_GE = -1

   def __listelementsdiffer(self, listname):
      for listitem in listname:
         if listname.count(listitem) > 1:
            return False
      else:
         return True

   def __init__(self, files, listofreaders = [], testsetname = '', solufilename = ''):
      self.files = files[0:len(files)]
      assert self.__listelementsdiffer(self.files)
      self.testruns = []
      self.readers = []
      self.datakeys = []
      for filename in files:
         testrun = TestRun(filename, solufilename, testsetname)
         self.testruns.append(testrun)
         testrun.settings = filename.split('.')[-2]
            
      self.datacollector = DataCollector()
      self.datacollector.registerListOfReaders(listofreaders)
      if solufilename != '':
#        print 'solufiledatacollector initialized for solufilename:', solufilename
         self.solufiledatacollector = SoluFileDataCollector()
        
      self.readers = listofreaders
      for reader in self.readers:
         self.addDataKey(reader.datakey)

   def addDataKey(self, datakey):
      self.datakeys.append(datakey)
    
   def setTestRun(self, testrun):
      self.datacollector.setTestRun(testrun)
      if self.solufiledatacollector != None:
         self.solufiledatacollector.setTestRun(testrun)

   def __makeProbNameList__(self):
      self.probnamelist = []
      for testrun in self.testruns:
         if self.probnamelist == []:
            self.probnamelist = testrun.problist
         else:
            if testrun.datacollected and testrun.problist == []:
               print testrun.getIdentification()
            assert not testrun.datacollected or testrun.problist != []
            for probname in testrun.problist:
               if not probname in self.probnamelist:
                  self.probnamelist.append(probname)
                    
                        
        

   def collectData(self):
      for testrun in self.testruns:
         self.setTestRun(testrun)
         self.datacollector.collectData()
         if self.solufiledatacollector != None:
            #print 'Collecting Solu File Data'
            assert self.solufiledatacollector.testrun == self.datacollector.testrun
            self.solufiledatacollector.collectData()
                
#      for testrun in self.testruns:
         #print testrun.problist
      self.__makeProbNameList__()
#        self.calculateData()

   def problemCompareData(self, probname, testrun1, testrun2, datakey, compsign):
      if not datakey in self.datakeys:
         raise self.datakey_err
            
#        if not testrun1.datacollected or not testrun2.datacollected:
#            print 'Collect all Data First'
        
      if not probname in self.probnamelist:
         raise self.probname_err

      data1 = float(testrun1.problemGetData(probname, datakey))
      data2 = float(testrun2.problemGetData(probname, datakey))

      return (data1 - data2) * compsign

   def getBestTestRunForProblem(self, probname, datakey, compsign):
      bestrun = None
      for testrun in self.testruns:
         if bestrun == None:
            bestrun = testrun
         elif self.problemCompareData(probname, bestrun, testrun, datakey, compsign) > 0:
            bestrun = testrun
      return bestrun

   def getBestRuns(self, datakey, compsign):
      bestruns = []
      for probname in self.probnamelist:
         bestruns.append(self.getBestTestRunForProblem(probname, datakey, compsign))
      return bestruns

   def excludeProb(self, probname, excludereasons=[]):
      for testrun in self.testruns:
         for reason in excludereasons:
            if Comparator.EXCLUDE_REASON_TIMEOUT == reason:
               if testrun.problemGetData(probname, TimeLimitReader.datakey) == TimeLimitReader.timelimit_reached:
                  return True
            elif Comparator.EXCLUDE_REASON_NOOPTIMALSOLUTIONKNOWN == reason:
               if testrun.problemGetSoluFileStatus(probname) != 'opt':
                  return True
            elif Comparator.EXCLUDE_REASON_NOSOLUTIONKNOWN == reason:
               if testrun.problemGetSoluFileStatus(probname) == 'unkn':
                  return True
            elif Comparator.EXCLUDE_REASON_ZEROSOLUTION == reason:
               if testrun.problemGetOptimalSolution(probname) == 0:
                  return True
            elif Comparator.EXCLUDE_REASON_INFEASIBLE == reason:
               if testrun.problemGetSoluFileStatus(probname) == 'inf':
                  return True
      else:
         return False

   def testrunGetKeyGeomMean(self, testrun, datakey, exclude=False):
      listofnumbers = []
      for probname in self.probnamelist:
         if not self.excludeProb(probname) or not exclude:
            listofnumbers.append(float(testrun.problemGetData(probname, datakey)))
      return Misc.listGetGeomMean(listofnumbers)

   def keyGetBestValue(self, probname, datakey, compsign):
      bestrun = self.getBestTestRunForProblem(probname, datakey, compsign)
      assert bestrun != None
      return bestrun.problemGetData(probname, datakey)

   def keyGetBestValues(self, datakey, compsign):
      values = []
      for probname in self.probnamelist:
         values.append(self.keyGetBestValue(probname, datakey, compsign))
      return values


   def testrunGetProbGapToOpt(self, testrun, probname):
      assert probname in testrun.problist
      optsol = testrun.problemGetOptimalSolution(probname)
      status = testrun.problemGetSoluFileStatus(probname)
      pb = testrun.problemGetData(probname, PrimalBoundReader.datakey)
      if status == 'opt' or status == 'best':
         return Misc.getGap(float(pb), float(optsol))
      else:
         return Misc.FLOAT_INFINITY

   def calculateData(self):
      self.calculateGaps()

   def calculateGaps(self):
      for probname in self.probnamelist:
         for testrun in self.testruns:
            gap = self.testrunGetProbGapToOpt(testrun, probname)
            data = (probname, Comparator.datakey_gap, '%8.2g'%(gap))
            testrun.addData(data)
      self.addDataKey(Comparator.datakey_gap)



# -*- coding: utf-8 -*-
__all__=[ "DataCollector",
          "Comparator",
          "Misc",
          "StatisticReader_GeneralInformationReader",
          "StatisticReader_PrimalBoundReader",
          "StatisticReader_ProblemNameReader",
          "StatisticReader",
          "StatisticReader_SoluFileReader",
          "StatisticReader_SolvingTimeReader",
          "StatisticReader_TimeLimitReader",
          "StatisticReader_PrimalBoundHistoryReader",
          "TestRun",
          ]


import os

from StatisticReader import StatisticReader

class ProblemNameReader(StatisticReader):
   '''
      reads a problem name
   '''
   name = 'ProblemNameReader'
   regular_exp = '@01'
   datakey = 'ProblemName'
   columnwidth = 40
   columnheaderstr = 'Problem Name'.ljust(columnwidth)
   extensions = ["mps", "cip", "fzn", "pip"]

   def extractStatistic(self, line):
      fullname = self.getSplitLineWithRegexp(self.regular_exp, line, index = 1, startofline = True)
      if fullname != None:
         namewithextension=os.path.basename(fullname);
         namewithextension = os.path.splitext(namewithextension)[0]
            
         if ".mps" in namewithextension[-4:]:
            namewithextension = namewithextension[:-4]
            
         if namewithextension.endswith('.lp'):
            namewithextension=namewithextension[:-3]
         #print namewithextension
         StatisticReader.setProblemName(namewithextension)
      return None




#!/usr/bin/env python3

helptext='''
Created on 06.01.2014

this parser generates php-readable data and static html out of faqtext.txt.

The format is strict, but very easy: The FAQ is separated into SECTIONs,
each of which may contain an arbitrary number of items, which consist (in this order)
of a QUESTION, an alpha-numeric, unique LABEL of your choice and
an ANSWER:

Example:
SECTION: General Questions about SCIP
    QUESTION: What is SCIP?
    LABEL:whatisscip
    ANSWER:
    <p>
      SCIP is a constraint integer program solver. It can be used as a framework for <b>branch-cut-and-price</b>
      and contains all necessary plugins to serve as a <b>standalone solver</b> for MIP, MINLP, and PBO.
    </p>
    ...

The example has the right format to go:
   - We have four keywords for sections, questions, labels and answers, resp.
   - an item first mentions a QUESTION:, which then gets a LABEL:, and finally, an ANSWER:. Do not mix up this order.
   - Each keyword ends with a colon ':' (!)
   - The label is a one-liner
   - line break after the 'ANSWER:'-tag
   - Questions and answers should be simple HTML text elements, do not provide specific classes etc. therein.

To make faq-generation for both home use and web use possible, extra-keywords should be used to identify
relative locations to the documentation and file extensions:
   - use PATHTODOC to denote relative paths to the documentation.
   - use LINKEXT to denote file-extensions.

The parser automatically interprets this.

The written faqdata.php can be transformed with php into static HTML by use of localfaq.php.
The written faq.inc produces the same output (without the need for php).

@author Gregor Hendel
@author Matthias Miltenberger
'''
import re
import sys

import argparse
parser = argparse.ArgumentParser(epilog=helptext)
parser.add_argument('--faqtextfile', type=str, default='faqtext.txt', help='faq text file')
parser.add_argument('--linkext', type=str, default='shtml', help='file extension for internal links')

sectiontag = "SECTION:"
questiontag = "QUESTION:"
answertag = "ANSWER:"
labeltag = "LABEL:"

faqinc_header='''<!-- this file is used to generate the local doxygen documentation -->
<!-- using make doc from within scip or soplex -->

<style>
@media (prefers-color-scheme: light) {
.reveal:hover {
    text-shadow: 1px 1px 1px #777;
}
.answer {
    background-color: #fff;
}
}
.answer {
    padding-left:   1em;
}
</style>

'''

def formatitem(item):
   '''
   returns a fully formatted php array containing all item information
   '''
   (question, answer, label) = item
   return """
          array(
              'question'=>'%s',
              'answer'=>'%s',
              'label'=>'%s'
             )""" % (question, answer, label)

def formatsection(sec):
   '''
   returns a fully formatted array to represent an entire section together with its items
   '''
   (section, items) = sec
   sectionlabel = "faq_{}".format(re.sub(r'[^a-zA-Z]', '', section).lower())
   return """
          array(
             'title'=>%s,
             'label'=>%s,
             'content'=>array(%s),
             )""" % (repr(section),
                     repr(sectionlabel),
                     ",\n".join(map(formatitem, items.__iter__())))

def formatallsections(sections, sectionitems):
   '''
   formats a list of sections
   '''
   return """
   $faq = array(
        %s
        );""" % (",\n".join([formatsection((section, sectionitems[section])) for section in sections]))

def write_faqinc(f, sections, sectiontimes):
   '''
   write main part of faq.inc
   '''

   # table of contents
   for section in sections :
      sectionlabel = "faq_{}".format(re.sub(r'[^a-zA-Z]', '', section).lower())
      f.write('<h3><a class="reveal_faq" href="#%s"><span class="fa fa-caret-right"></span> %s</a></h3>' % (sectionlabel, section));
      f.write('<ol>')
      for item in sectionitems[section] :
         question = item[0]
         label = item[2]
         question = re.sub(r"\\'", r"'", question)
         f.write('  <li>\n')
         f.write('    <a class="reveal_faq" href="#%s">\n' % label)
         f.write('      %s' % question)
         f.write('    </a>\n')
         f.write('  </li>\n  ')
      f.write('</ol>\n')
   f.write('<hr />\n')

   for section in sections :
      sectionlabel = "faq_{}".format(re.sub(r'[^a-zA-Z]', '', section).lower())
      f.write('<h3 id="%s" class="anchor">' % sectionlabel)
      f.write('<span class="fa fa-caret-right"></span> %s<a href="#" class="pull-right"><span title="go to top" class="fa fa-caret-up"></span></a></h3><ol>' % section)
      for item in sectionitems[section] :
         question = item[0]
         answer = item[1]
         # this undoes the replacement done in notagline = re.sub(r"'", r"\'", notagline) below
         # if the PHP output is dropped, then we could clean this up
         question = re.sub(r"\\'", r"'", question)
         answer = re.sub(r"\\'", r"'", answer)
         label = item[2]
         f.write('  <li id="%s" class="anchor">\n' % label)
         f.write('    <h4>\n')
         f.write('      <a class="reveal_faq" href="#%s">%s</a>\n\n' % (label, question))
         f.write('      <a href="#" class="pull-right"><span class="fa fa-caret-up" title="go to top"></span></a>\n')
         f.write('    </h4>\n')
         f.write('    <div id="%s_ans">\n' % label)
         f.write('      %s' % answer)
         f.write('    </div>\n')
         f.write('  </li>\n  ')
      f.write('</ol>\n')


if __name__ == '__main__':
   '''
   main execution method of script.

   reads an faqtext.txt and creates the corresponding faqdata.php php-readable data file
   '''
   sectionitems = {}
   sections = []
   items = []
   mode = None
   currquestion = curranswer = currlabel = None
   args = parser.parse_args()
   substitutions = {"LINKEXT":args.linkext,
                    "PATHTODOC": "."}
   faqtext = args.faqtextfile
   # open the faqtext file and iterate over all lines. change mode depending on last seen tag
   with open(faqtext, 'r') as currentfile:
      for line in currentfile:
         notagline = line
         for tag in [sectiontag, questiontag, answertag, labeltag]:
            notagline = re.sub(r"%s" % (tag), "", notagline, flags=re.MULTILINE).strip()

         notagline = re.sub(r"'", r"\'", notagline)

         # use the substitutions to get working links
         for key, val in substitutions.items():
             notagline = re.sub(key, val, notagline)
         # print notagline
         if sectiontag in line:

            if mode is not None:
               items.append((currquestion, curranswer, currlabel))
               sectionitems[sections[-1]] = items[:]

            sections.append(notagline)
            mode = None
            items = []

         elif questiontag in line:
            if mode == answertag:
               items.append((currquestion, curranswer, currlabel))
            currquestion = notagline
            mode = questiontag
         elif labeltag in line:
            currlabel = notagline
            mode = labeltag
         elif answertag in line:
            curranswer = None
            mode = answertag
         else:

            if mode == questiontag:
               currquestion += "\n%s" % notagline
            elif mode == answertag:
               if curranswer is None:
                  curranswer = "%s" % (notagline)
               else:
                  curranswer = "%s\n%s" % (curranswer, notagline)

      else:
         if mode is not None:
            items.append((currquestion, curranswer, currlabel))
            sectionitems[sections[-1]] = items[:]
            # print "Keys:" , sectionitems.keys()
            f = open("faqdata.php", 'w')
            f.write("<?php \n%s\n ?>" % formatallsections(sections, sectionitems))
            f.close()
            f = open("faq.inc", 'w')
            f.write(faqinc_header)
            write_faqinc(f, sections, sectionitems)
            f.close()



# -*- coding: utf-8 -*-


class StatisticReader:
   '''
   base class for all statistic readers - readers should always inherit from this base class for minimal implementation
   effort
   
   readers only need to overwrite the methods extractStatistic() and perhaps execEndOfProb() 
   '''

   name = 'NO_NAME_DEFINED_YET'
   regular_exp = '???###!!!'
   INSTANCE_END_EXPRESSION = '=ready='
   datakey = 'NO KEY'
   #print data#
   columnwidth = -1
   columnheaderstr = 'NOCOLHEAD'
   problemname = ''
   problemnamelist = []

   noregexp_err = 'no reg exp defined yet'
   
   
   # the reader might behave differently depending on the solver type, due to the different output
   SOLVERTYPE_SCIP="SCIP"
   SOLVERTYPE_GUROBI="GUROBI"
   SOLVERTYPE_CPLEX="CPLEX"
   SOLVERTYPE_CBC="CBC"
   SOLVERTYPE_XPRESS="XPRESS"
   solvertype = SOLVERTYPE_SCIP

   def setProblemName(problemname):
      if problemname in StatisticReader.problemnamelist:
         counter = StatisticReader.problemnamelist.count(problemname)
         StatisticReader.problemname = problemname+'_'+repr(counter)
      else:
         StatisticReader.problemname = problemname
      StatisticReader.problemnamelist.append(problemname)
       
   setProblemName = staticmethod(setProblemName)

   def initializeForTestrun(self):
      StatisticReader.problemnamelist = []

   def getSplitLineWithRegexp(self, regular_exp, line, index = -1, startofline=False):
      length=len(regular_exp)
      tmpline = line
      if startofline:
         tmpline = line[0:length]
      list=[]
      if regular_exp in tmpline:
         list = line.split()
      else:
         return None

      if index == -1:
         return list
      else:
         assert 0 <= index
         assert index < len(list)
         return list[index]

      return None
       
   def getName(self):
      return self.name

   def endOfInstanceReached(self, line):
      if StatisticReader.INSTANCE_END_EXPRESSION in line:
         return True
      else:
         return False
   
   def extractStatistic(self, line):
      '''
      overwrite this method for own reader subclasses - make sure that a return statement different from 'None'
      is only accepted once on each instance of the log file
      '''
      return None
       
   def checkSolverType(self, line):
      changed = False
      if "Gurobi Optimizer version" in line and self.solvertype != StatisticReader.SOLVERTYPE_GUROBI:
         StatisticReader.solvertype = StatisticReader.SOLVERTYPE_GUROBI
         changed = True
      elif "SCIP version " in line and self.solvertype != StatisticReader.SOLVERTYPE_SCIP:
         StatisticReader.solvertype = StatisticReader.SOLVERTYPE_SCIP
         changed = True
      elif "Welcome to IBM(R) ILOG(R) CPLEX(R) Interactive Optimizer" in line and self.solvertype != StatisticReader.SOLVERTYPE_CPLEX:
         StatisticReader.solvertype = StatisticReader.SOLVERTYPE_CPLEX
         changed = True
      elif "Welcome to the CBC MILP Solver" in line and self.solvertype != StatisticReader.SOLVERTYPE_CBC:
         StatisticReader.solvertype = StatisticReader.SOLVERTYPE_CBC
         changed = True
      elif "FICO Xpress Optimizer" in line and self.solvertype != StatisticReader.SOLVERTYPE_XPRESS:
         StatisticReader.solvertype = StatisticReader.SOLVERTYPE_XPRESS
         changed = True
      if changed:
         print "changed solver type to", StatisticReader.solvertype
          

   def setTestRun(self, testrun):
      pass

   def execEndOfProb(self):
      '''
      overwrite this method to implement final behaviour at the end of each problem, such as setting flags
      '''
      return None

   def operateOnLine(self, line):
      
      if self.endOfInstanceReached(line):
         stat = self.execEndOfProb()
         if stat != None:
            assert StatisticReader.problemname != ''
            assert self.datakey != 'NO KEY'
#            print ' returning Statistic ', (StatisticReader.problemname, self.datakey, stat)
            return (StatisticReader.problemname, self.datakey, stat)
         else:
            return None
      else:
         self.checkSolverType(line)
         stat = self.extractStatistic(line)
         if stat != None:
            assert StatisticReader.problemname != ''
            assert self.datakey != 'NO KEY'
#            print ' returning Statistic ', (StatisticReader.problemname, self.datakey, stat)
            return (StatisticReader.problemname, self.datakey, stat)
         else:
            return None



from StatisticReader import StatisticReader

class PrimalBoundReader(StatisticReader):
   name = 'PrimalBoundReader'
   regular_exp = 'Primal Bound       :'
   datakey = 'PrimalBound'
   columnwidth = 18
   columnheaderstr = 'Primal Bound'.rjust(columnwidth)
   maxndigits = 3

   def extractStatistic(self, line):
      if self.regular_exp in line:
         fullline=line.split()
         pb = float(fullline[3])
         return pb
      else:
         return None



from StatisticReader import StatisticReader
import re

class TimeLimitReader(StatisticReader):
   '''
   extracts the time limit for an instance
   '''
   name = 'TimeLimitReader'
   solvingtimereadkeys = {StatisticReader.SOLVERTYPE_SCIP : "Solving Time (sec) :",
                  StatisticReader.SOLVERTYPE_CPLEX : "Solution time =",
                  StatisticReader.SOLVERTYPE_GUROBI : "Explored ",
                  StatisticReader.SOLVERTYPE_CBC : "Coin:Total time (CPU seconds):",
                  StatisticReader.SOLVERTYPE_XPRESS : " *** Search "}
                   
   timelimitreadkeys= {
                  StatisticReader.SOLVERTYPE_SCIP : '(SCIP> limits/time =|SCIP> set limits time)',
                  StatisticReader.SOLVERTYPE_CPLEX : 'CPLEX> New value for time limit in seconds',
                  StatisticReader.SOLVERTYPE_GUROBI : "Changed value of parameter TimeLimit to",
                  StatisticReader.SOLVERTYPE_CBC : "Coin:seconds has value",
                  StatisticReader.SOLVERTYPE_XPRESS : " @05"}
                   
   solvingtimelineindex = {
                  StatisticReader.SOLVERTYPE_SCIP : -1,
                  StatisticReader.SOLVERTYPE_CPLEX : 3,
                  StatisticReader.SOLVERTYPE_GUROBI : -2,
                  StatisticReader.SOLVERTYPE_CBC : 4,
                  StatisticReader.SOLVERTYPE_XPRESS : 5}
   columnwidth = 20
   datakey = 'TimeLimit'
   columnheaderstr = 'Timelimit'.rjust(columnwidth)
   #timelimit = 7200.00
   timelimit_reached = 'true'
   timelimit_not_reached = 'false'

   
   def __init__(self):
      self.timelimit=7200

   def extractStatistic(self, line):
        
      if re.search(self.timelimitreadkeys[StatisticReader.solvertype], line):
         self.timelimit = float(line.split()[-1])
      elif self.solvingtimereadkeys[StatisticReader.solvertype] in line:
         self.solvingtime = float(line.split()[self.solvingtimelineindex[StatisticReader.solvertype]])
      return None

   def execEndOfProb(self):
      return self.timelimit
   
   
   



# -*- coding: utf-8 -*-
from StatisticReader import StatisticReader

class GeneralInformationReader(StatisticReader):
   '''
      this reader extracts general information from a log file, that is, the solver version, the used LP solver
      in case of SCIP and the settings
   '''
   name = 'GeneralInformationReader'
   regular_exp = ['loaded parameter file', 'default.set']
   columnwidth = 20
   actions = {}
   testrun = None
   columnheaderstr = 'Version'.rjust(columnwidth)
   
   versionkeys = {StatisticReader.SOLVERTYPE_SCIP : 'SCIP version',
                  StatisticReader.SOLVERTYPE_CPLEX : "Welcome to IBM(R) ILOG(R) CPLEX(R)",
                  StatisticReader.SOLVERTYPE_GUROBI : "Gurobi Optimizer version",
                  StatisticReader.SOLVERTYPE_CBC : "Version:",
                  StatisticReader.SOLVERTYPE_XPRESS : "FICO Xpress Optimizer"}
                  
   versionlineindices = {StatisticReader.SOLVERTYPE_SCIP : 2,
                         StatisticReader.SOLVERTYPE_CPLEX : -1,
                         StatisticReader.SOLVERTYPE_GUROBI : -1,
                         StatisticReader.SOLVERTYPE_CBC : -1,
                         StatisticReader.SOLVERTYPE_XPRESS: 4}
   
   def __init__(self, testrun = None):
      self.testrun = testrun
   
   def setTestRun(self, testrun):
      self.testrun = testrun
   
   def extractStatistic(self, line):
      if self.testrun == None:
         return None
         
      if self.testrun.version == '' and self.versionkeys[StatisticReader.solvertype] in line:
         if self.testrun.solver == '':
            self.testrun.solver = StatisticReader.solvertype
            
         if StatisticReader.solvertype == StatisticReader.SOLVERTYPE_SCIP:
            self.__handleVersion(line)
         else:
            self.testrun.version = line.split()[self.versionlineindices[StatisticReader.solvertype]]
      elif self.testrun.settings == '' and StatisticReader.solvertype == StatisticReader.SOLVERTYPE_SCIP:
         if 'default.set' in line:
            self.testrun.settings = 'default'
         elif 'loaded parameter file' in line:
            splittedsettings = self.getSplitLineWithRegexp('loaded parameter file', line, index=3, startofline = True);
            if splittedsettings != None:
               splittedsettings = splittedsettings.split('/')
               settings = splittedsettings[- 1].split('.set')[0]
               self.testrun.settings = settings
      
      elif self.testrun.settings == '' and StatisticReader.solvertype == StatisticReader.SOLVERTYPE_CPLEX:
         if "CPLEX> Non-default parameters written to file" in line:
            self.testrun.settings=line.split('.')[-3]
            print line.split('.')
         
             
      return None
      
   def __handleVersion(self, line):
      
      splittedline = self.getSplitLineWithRegexp('SCIP version', line)
      if splittedline != None:
         version = splittedline[2]
         if self.testrun.version != '':
            assert self.testrun.version == version
         else:
            self.testrun.version = version
         
         lpsolver = splittedline[splittedline.index('[LP') + 2].rstrip(']')
         if self.testrun.lpsolver != '':
            assert self.testrun.lpsolver == lpsolver
         else:
            self.testrun.lpsolver = lpsolver

















    



import re


class Snippet:
    def __init__(self, tag, start, end):
        self.tag = tag
        self.start = start
        self.end = end

    def matches_start(self, string):
        return self.start is not None and string.startswith(self.start)

    def matches_end(self, string):
        return self.end is not None and string.startswith(self.end)

    def get_tag(self):
        return "/**! [Snippet%s] */" % self.tag


def handle_line(line, snippet, target_file):
    start_pattern_matches = snippet.matches_start(line)
    end_pattern_matches = snippet.matches_end(line)

    if start_pattern_matches or end_pattern_matches:
        target_file.write("%s\n" % snippet.get_tag())

    return end_pattern_matches


none_snippet = Snippet("", None, None)

snippet_list = [
    Snippet("Version", "SCIP version", "SCIP> help"),
    Snippet("Help", "SCIP> help", "SCIP> read check/instances/MIP/stein27.fzn"),
    Snippet("Opt1", "SCIP> read check/instances/MIP/stein27.fzn", "SCIP> write solution stein27.sol"),
    Snippet("WriteSolutions", "SCIP> write solution stein27.sol", "SCIP> set save settingsfile.set"),
    Snippet("SaveSettingsOverview", "SCIP> set save settingsfile.set", "SCIP> display heuristics"),
    Snippet("DisplayStatistics", "SCIP> display heuristics", "SCIP> set"),
    Snippet("SetSettings", "SCIP> set", "SCIP> set default"),
    Snippet("Opt2", "SCIP> set default", "SCIP> set save settingsfile.set"),
    Snippet("SaveSettingsFull", "SCIP> set save settingsfile.set", "SCIP> set diffsave settingsfile.set"),
    Snippet("SaveSettingsDiff", "SCIP> set diffsave settingsfile.set", "SCIP> set load settingsfile.set"),
    Snippet("LoadSettings", "SCIP> set load settingsfile.set", "SCIP> quit")
]


def main():
    with open("inc/shelltutorial/shelltutorialannotated.tmp", "w") as annotated_file:
        with open("inc/shelltutorial/shelltutorialraw.tmp", "r") as raw_file:
            snippet_iterator = iter(snippet_list)
            current_snippet = next(snippet_iterator)
            for line in raw_file:

                while handle_line(line, current_snippet, annotated_file):
                    try:
                        current_snippet = next(snippet_iterator)
                    except StopIteration:
                        current_snippet = none_snippet

                annotated_file.write(line)


if __name__ == "__main__":
    main()



#! /usr/bin/env python

# This script allows to run PySCIPOpt using the SCIP make test environment.
# usage: make test[cluster] EXECUTABLE=check/pyscipopt-runner.py PYTHON=python [options]
#
# You may use this file as template to extend it and add custom code.
# The executable needs to have the .py file extension to be recognized by
# the check scripts and _must_ be an executable file.
#
# It should _not_ be necessary to modify parse_tmp_file() nor build_model(),
# unless you need to add a plugin before the problem instance is read.

from pyscipopt import Model
import os
import sys

def parse_tmp_file(tmpfile):
   """helper method to parse TMPFILE that has been set up by configuration_tmp_setup_scip.sh"""
   instance = ""
   settings = ""
   outsettings = ""
   params = []
   parammap = {
      "misc referencevalue" : ("misc/referencevalue", "f"),
      "limits time" : ("limits/time", "f"),
      "limits nodes" : ("limits/nodes", "lld"),
      "limits memory" : ("limits/memory", "f"),
      "lp advanced" : ("lp/threads", "d"),
      "timing clocktype" : ("timing/clocktype", "d"),
      "display freq" : ("display/freq", "d"),
      "memory savefac" : ("memory/savefac", "f")
    }

   for l in tmpfile:
      splitstr = l.split()
      # instance
      if l.startswith("read"):
         instance = splitstr[1]
      # input settings
      elif l.startswith("set load"):
         settings = splitstr[2]
      # output settings
      elif l.startswith("set save"):
         outsettings = splitstr[2]
      # SCIP parameters
      elif l.startswith("set"):
         p = splitstr[1] + " " + splitstr[2]
         assert p in parammap
         img = parammap[p]
         val = splitstr[-1]
         # store parameters as triples of the form (name,type,value)
         params.append((img[0], img[1], val))

   return {"instance" : instance, "settings" : settings, "outsettings" : outsettings, "params" : params}


def build_model():
   """builds a model, reads input data provided by the scripts, and writes out setting file"""
   # generate a model
   model = Model()

   # parse TMP file
   data = parse_tmp_file(sys.stdin)

   # read settings file
   model.readParams(data["settings"])

   # set parameters
   for (name,t,val) in data["params"]:
      if t == "f":
         model.setRealParam(name, float(val))
      elif t in "d":
         model.setIntParam(name, int(val))
      elif t == "lld":
         model.setLongintParam(name, int(val))

   # read problem
   model.readProblem(data["instance"])

   # prepare model
   prepare_model(model, data)

   # print version, optimize, and print statistics
   model.printVersion()

   # write parameters
   model.writeParams(data["outsettings"])

   return model

#
# example main
#
if __name__ == "__main__":

   model = build_model()

   # optimize and print statistics
   model.optimize()
   model.printStatistics()



# -*- coding: utf-8 -*-
from math import *

'''
   Various methods for evaluation such as gap calculation, geometric means etc. and some printing methods
'''
FLOAT_INFINITY = 1e20
FLOAT_LARGE = 1e15
INT_INFINITY = 1e09
DEFAULT_MIN_GEOM_MEAN = 1.0


def getSoluFileProbName(probname):
   return probname.split('.')[0]

def floatPrint(number, digitsbef=16, digitsafter=9, dispchar='g' ):
   formatstring='%'+repr(digitsbef) + '.' + repr(digitsafter) + dispchar
   if number >= FLOAT_INFINITY:
      return 'Inf'
   else:
      return formatstring % (number)
        
def getTexName(name):
   return name.replace('_', '\_')
           

def getGap(value, referencevalue, useCplexGap=False):
   '''
   calculate the gap between value and reference value in percent. Gap is either calculated
   w.r.t the referencevalue, i.e., abs(value-referencevalue)/abs(referencevalue) * 100,
   or in 'Cplex'-fashion, that is,  abs(value-referencevalue)/max(abs(referencevalue), abs(value)) * 100
   '''
   if not useCplexGap:
      if referencevalue == 0.0:
         if value == 0.0:
            return 0.0
         else:
            return FLOAT_INFINITY
      elif referencevalue == FLOAT_INFINITY or value == FLOAT_INFINITY:
         return FLOAT_INFINITY
      else:
         return abs( value - referencevalue )/float(abs(referencevalue)) * 100
   else: # use the CPLEX gap
      if value == FLOAT_INFINITY:
         return FLOAT_INFINITY
      maximum = max(abs(value), abs(referencevalue))
      if maximum <= 10e-9:
         return 0.0
      else:
         return abs(value - referencevalue) / maximum * 100


def listGetArithmeticMean(listofnumbers):
   '''
   returns the arithmetic mean of a list of numbers
   '''
   arithmeticmean = sum(listofnumbers)
   arithmeticmean /= max(len(listofnumbers),1)*1.0
   return arithmeticmean
   
def listGetGeomMean(listofnumbers, mingeommean=DEFAULT_MIN_GEOM_MEAN):
   '''
   returns the geometric mean of a list of numbers, where each element under consideration
   has value min(element, mingeommean)
   '''
   geommean = 1.0
   nitems = 0
   for number in listofnumbers:
      nitems = nitems + 1
      nextnumber = max(number, mingeommean)
      geommean = pow(geommean, (nitems - 1)/float(nitems)) * pow(nextnumber, 1 / float(nitems)) 
   return geommean
   
def listGetShiftedGeometricMean(listofnumbers, shiftby=10.0):
   '''
   returns the shifted geometric mean of a list of numbers, where the additional shift defaults to 
   10.0 and can be set via shiftby
   '''
   geommean = 1.0
   nitems = 0
   for number in listofnumbers:
      nitems = nitems + 1
      nextnumber = number + shiftby
      geommean = pow(geommean, (nitems - 1)/float(nitems)) * pow(nextnumber, 1 / float(nitems)) 
   return geommean - shiftby
                
   
def cutString(string, char = '_', maxlength = -1):
   iscuttable = True
   stringcopy = string[0:len(string)]
   while len(stringcopy) > maxlength and iscuttable:
      iscuttable = False
      listofstrings = stringcopy.split(char)
      maxlen = 2
      index = -1
      for stringpart in listofstrings:
         if len(stringpart) >= 3 and len(stringpart) > maxlen:
            index = listofstrings.index(stringpart)
            maxlen = len(stringpart)
            iscuttable = True
      else:
         if iscuttable:
            stringtocut = listofstrings[index]
            assert len(stringtocut) >= 3
            listofstrings[index]= stringtocut[0] + stringtocut[len(stringtocut)-1]
            stringcopy = ''
            for stringitem in listofstrings:
               stringcopy = stringcopy + stringitem + char
            else:
               stringcopy.rstrip(char)
            
   return stringcopy.rstrip(char)

def listGetMinColWidth(listofitems):
   maxlen = -1
   for item in listofitems:
      if len(item) > maxlen:
         maxlen = len(item)
   return maxlen
   
   



#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys
import commands

instances = []
paths = {}

def main():
    if len(sys.argv) < 4:
        print 'usage: mergetestsets.py <.test file 1> <.test file 2> <location of merged .test file>'
        return 0

    infile1 = file(sys.argv[1], "r")
    infile2 = file(sys.argv[2], "r")
    outfile = file(sys.argv[3], "w")
    solufile = file(sys.argv[3].replace(".test","")+".solu", "w")

    for line in infile1.readlines() + infile2.readlines():
        if line.startswith("#"):
            continue
        else:
            instance = line.split("/")[-1].strip().replace(".mps","").replace(".lp","").replace(".gz","")

            if not instance in instances:
                instances.append(instance)
                paths[instance] = line.strip()
            else:
                if paths[instance] != line.strip():
                    print "instance %s with two different paths:\n\t%s\n\t%s" %(instance, paths[instance], line.strip())

    for instance in sorted(instances):
        outfile.write("%s\n" %(paths[instance]))

        grepline = commands.getoutput("grep '%s ' %s %s" %(instance, sys.argv[1].replace(".test","")+".solu", sys.argv[2].replace(".test","")+".solu"))

        if grepline.strip() == "":
            grepline = commands.getoutput("grep '%s$' %s %s" %(instance, sys.argv[1].replace(".test","")+".solu", sys.argv[2].replace(".test","")+".solu"))

        line1 = grepline.strip().split("\n")[0].split(":",1)[1]

        if len(grepline.strip().split("\n")) != 1:
            assert len(grepline.strip().split("\n")) == 2, grepline
            line2 = grepline.strip().split("\n")[1].split(":",1)[1]

            if line1.split()[0] != line2.split()[0]\
               or (line1.split()[0] == "=opt=" and \
                   abs(float(line1.split()[2]) - float(line2.split()[2])) > 1e-04):
                print "different solution file entries:\n" + grepline

        solufile.write(line1+"\n")

if __name__ == "__main__":
        sys.exit( main() )



from DataCollector import DataCollector
from StatisticReader_SoluFileReader import SoluFileReader

class SoluFileDataCollector(DataCollector):
   '''
      a data collector to be run on .solu files.
   '''
      
   def __init__(self, testrun = None, index = 0):
      self.registerReader(SoluFileReader())
      if testrun != None:
         self.setTestrun(testrun)
         self.filestring = self.testrun.solufilename
      
   def setTestRun(self, testrun):
      if self.testrun != testrun:
         self.testrun = testrun
         self.filestring = self.testrun.solufilename
         for reader in self.listofreaders:
            reader.setTestRun(testrun)
         self.datacollected = False



#!/usr/bin/python
# -*- coding: utf-8 -*-
## import all required classes from the /src/ folder 
## - Note: Classes which are created in the /src/ folder have to be declared public in the file src/__init__.py
##         before they can be used here
from primalintegral_src import * 

## import system-related I/O to have access to terminal commands etc.
import sys
import numpy as np
import os

'''This is the main script to be called to generate Primal Integrals.
 
  The script reads .out files generated by our 'make test(cluster)(gurobi,cpx,xpress,cbc)'
  and needs a .solu file for optimal or best known solutions to base the integral on
  It will generate two .tex source files to contain instancewise integrals and average integrals
  to files specified by two variables, 'integralsfilename' and 'meanintegralsfilename', respectively.
  These can be specified by modifying the script or via command line arguments.
  
  An empty .solu-file will cause the program to crash
  
  Note that all instances which are known to be infeasible are automatically excluded from the primal integral
  calculation.
  
  Example calls:
  1)
  python primalintegral.py -files ../PATH/TO/check.TESTSET.*.out -solufilename ../PATH/TOSOLUFILE/TESTSET.solu
  
  will run over all .out log files matching the expression and obtain the Solution information from the specified
  solufilename and write the integrals and plots to 'integrals.tex' and 'meanintegrals.tex' to this directory
  
  
  2)
  python primalintegral.py -files ../PATH/TO/check.TESTSET.*.out -solufilename ../PATH/TOSOLUFILE/TESTSET.solu
      -integralfilesname PATH/TO/somefilename.tex -meanintegralsfilename PATH/TO/someotherfilename.tex
      
   will generate the .tex output in the specified files instead  
    
'''


## rename the classes for a quicker access
DataCollector = DataCollector.DataCollector
GeneralInformationReader = StatisticReader_GeneralInformationReader.GeneralInformationReader
PrimalBoundHistoryReader = StatisticReader_PrimalBoundHistoryReader.PrimalBoundHistoryReader
PrimalBoundReader = StatisticReader_PrimalBoundReader.PrimalBoundReader
ProblemNameReader = StatisticReader_ProblemNameReader.ProblemNameReader
#PrimalBoundAfterRootNodeReader = StatisticReader_PrimalBoundAfterRootNode.PrimalBoundAfterRootNodeReader

SoluFileReader = StatisticReader_SoluFileReader.SoluFileReader
SolvingTimeReader = StatisticReader_SolvingTimeReader.SolvingTimeReader
StatisticReader = StatisticReader.StatisticReader
TimeLimitReader = StatisticReader_TimeLimitReader.TimeLimitReader

TestRun = TestRun.TestRun
Comparator = Comparator.Comparator

###  the list of .out files evaluated by this script. Usually entered by typing '-files', followed by the .out-files to be part of the evaluation
filelist = [
#   "../new-scip-git/check/results/check.short.scip-3.0.1.1.linux.x86_64.gnu.opt.cpx.opt55.default.out",
#   "../new-scip-git/check/results/check.short.scip-3.0.1.1.linux.x86_64.gnu.opt.cpx.opt54.proxyunlimited.out"
]

#solufilename = '../../scip-git/check/testset/MMM.solu'
solufilename = ''

EXECUTE_PLUGIN = True

### exclude problems manually by explicitly naming them here
excludeProbs = [#"ns1758913", "timetab1", "timetab2", "momentum2" 
#"snp-001-04.lp"
]

# # the timelimit to be taken as default timelimit
DEFAULT_TIMELIMIT = 36000

### enter the list of exclude reasons here. Every instance matching one of the listed reasons like, e. g., 
### it is considered infeasible, will be excluded from the evaluated set of instances, if 'Comparator.EXCLUDE_REASON_INFEASIBLE'
### is an element of this list.
excludereasons = [Comparator.EXCLUDE_REASON_INFEASIBLE,Comparator.EXCLUDE_REASON_NOSOLUTIONKNOWN]

options = ['-files', '-solufilename', '-meanintegralsfilename', '-integralsfilename', '-timelimit']

# ## choose the names of the LaTeX files to be generated for the geometric mean and the explicit instance wise table, respectively
meanintegralsfilename = 'meanintegrals.tex'
integralsfilename = 'integrals.tex'

## initialize default names
testsetname = ''
## install all required readers in this list. Remove readers which are not required for your specific task to speed up the evaluation process
listofreaders = [
                 GeneralInformationReader(),
                 PrimalBoundHistoryReader(),
                 PrimalBoundReader(),
                 ProblemNameReader(),
                 SolvingTimeReader(), 
                 TimeLimitReader(), 
                 ]

datakeys = []  ### initialize the data key list
lastoption = None

### read all arguments from the command line 
for eacharg in sys.argv:
#    print eacharg
   if eacharg in options:
      lastoption = eacharg
   elif lastoption == '-files':
      filelist.append(eacharg)
   elif lastoption == '-solufilename':
      solufilename = eacharg
   elif lastoption == '-timelimit':
      DEFAULT_TIMELIMIT = float(eacharg)
   elif lastoption == '-meanintegralsfilename':
      meanintegralsfilename = eacharg
   elif lastoption == '-integralsfilename':
      integralsfilename = eacharg

execute = True ### The 'execute' flag prevents the script from being run in case of an invalid data set.


writemeanintegrals=True
# # the maximum primal-optimal gap considered for the evaluation
cutoffGap = 100;

# # 
usexlogaxis = False
useylogaxis = False


def changeNameForSoluFile(nameOfProb): 
   '''workaround to get access to .solu-file data even in case of corrupted instance names. 
   
      Keyword arguments:
      nameofProb -- the name of the instance to be potentially modified
      
   '''
   return nameOfProb
      
def getXFloat(somefloat):
   if usexlogaxis:
      return np.log10(max(1.0, somefloat))
   else:
      return somefloat

def getYFloat(somefloat):
   if useylogaxis:
      return np.log10(max(1.0, somefloat))
   else:
      return somefloat
      
def calcIntegralValue(thedatalist, optSol, cutoffgap, timelimit=DEFAULT_TIMELIMIT): 
   '''
      calculates the value of the primal-optimal integral over time. 
      
      Keyword arguments:
      thedatalist -- a list of string tuples (time,pb) representing the improvement of the incumbent objective pb over time
      optSol -- the value of an optimal or best known solution for the calculation of the gap
      cutoffgap -- an upper bound to the value of an evaluated gap
      timelimit -- the time limit as an upper bound for the time-abscissa (default DEFAULT_TIMELIMIT)
   '''
   
   if thedatalist == []:  
      return getXFloat(timelimit) * getYFloat(cutoffgap)
   else:
      integralVal = 0.0
      lastPointInTime = 0.0
      lastgap = getYFloat(cutoffgap)
      for element in thedatalist:
         time, pb = element
         time = getXFloat(min(float(time), timelimit))
         pb = float(pb)
         nextgap = getYFloat(min(Misc.getGap(pb, optSol, useCplexGap=True), cutoffgap))
         
         integralVal += (time - lastPointInTime) * lastgap
         lastgap = nextgap
         lastPointInTime = time
      else:
         # if the end of the data list has been reached, but a gap remains, the integral value is completed
         # by adding the rectangle until the timelimit
         if lastgap > 0.0:
            integralVal += lastgap * (getXFloat(timelimit) - lastPointInTime)
            
      return integralVal
      
      
      
def getMaxTime(probname, testruns):
   '''
      returns the maximum solving time by which a test run found a new solution for a specific instance
      
      Keyword arguments:
      probname -- the name of the instance
   '''
   themax = 0.0
   for testrun in testruns:
      thelist = testrun.problemGetData(probname, PrimalBoundHistoryReader.datakey)
      if thelist != []:
         time = thelist[- 1][0]
         themax = max(themax, time)
   return themax
   
def getMaxGap(probname, testruns, optimum):
   '''
      returns the maximum primal optimal gap of a solution objective for a specific instance
      
      Keyword arguments:
      probname -- the name of the instance
   '''
   themax = 0.0
   
   for testrun in testruns:
      thelist = testrun.problemGetData(probname, PrimalBoundHistoryReader.datakey)
      if thelist != []:
         bound = thelist[0][1]
         gap = min(Misc.getGap(bound, optimum, useCplexGap=True), cutoffGap)
         themax = max(themax, gap)
   return themax            



def getTexHeader(settingsForGeometry=''):
   '''
      returns a possible LaTeX-file header as string object. Settings for the geometry package are an optional argument
      
      Keyword arguments:
      settingsForGeometry -- special package options for the geometry package,e.g., 'twoside,bottom=2cm' (default '')
   '''
   return ('''\documentclass[titlepage, oneside, 10pt]{book}
 \\usepackage[utf8x]{inputenc}
 \\usepackage{amsmath}
 \\usepackage{amssymb}
 \\usepackage{amsthm}
\\usepackage{paralist}
\\usepackage{booktabs}
\\usepackage{overpic}
\\usepackage{color}
\\usepackage{pgfplots}
\\usepackage{tikz}
\\usepackage{pgfplotstable}
 \\usepackage{booktabs}
 \\usepackage{array}
 \\usepackage{colortbl}
\\usepackage{longtable}
\\usepackage{pdflscape}
\\usepackage{url}%s\n''' % defineGeometry(settingsForGeometry))
# \\usepackage[a4paper,bindingoffset=0.5cm,twoside,includehead,bottom=2cm]{geometry} 


def getTexAuthor():
   '''
      returns the author of the document
   '''
   return '''\\author{Gregor Hendel \\thanks{Konrad Zuse Institut f\"ur Informationstechnik Berlin,
               hendel@zib.de}}'''
               

def getTexPgfplotsset(settingspath, settings=''):
   '''
      returns a string which can be used in a LaTeX source file as PgfPlots set up. Use the empty string '' as settings to 
      have  a set of standard options for the settings path, or modify the settings.
      
      Keyword arguments:
      settingspath -- the Pgf path to store this setting as settingspath/.style, use e.g. settingspath='every axis'
                       to manipulate all plots in an axis environment.
      settings -- a string with PgfPlots style options for the specified Pgf path
      
   '''
   if settings == '':
      return '''\\pgfplotsset{
      %s/.style={
      width = \\linewidth,
      height = 5cm,
      xmin = 0,
      max space between ticks=50,
      ymin = 0,
      xlabel=Time(sec.),
      ylabel= Gap,
      axis x line=bottom,
      axis y line=left,
      const plot
      }
      }\n''' % settingspath
   else:
      return '''\\pgfplotsset{
      %s/.style={
      %s
      }
      }\n''' % (settingspath, settings)


def getTexSection(title):
   '''
      returns a string to open a new LaTeX Section.
      
      Keyword arguments:
      title -- the title of the section
   '''
   return "\\section{%s}\n" % Misc.getTexName(title)  

def defineGeometry(settingsForGeometry):
   '''
      returns a string which is used as definition for the geometry package in the method 'getTexHeader(...)'
      
      Keyword arguments:
      settingsForGeometry -- a string with package options for the geometry-package like, e.g., 'a4paper,bindingoffset=0.5cm,twoside'
   '''
   return '\\usepackage[%s]{geometry}\n' % settingsForGeometry


def getAxisPlot(dictOfPlots, dictOfSettings, settings='', legend=False):
   '''
      returns a string which can be embedded in a LaTeX file and produces a complete TikZ picture where all requested plots
      are drawn in a single axis environment. 
      
      Keyword arguments:
      dictOfPlots -- a dictionary of all plots in which each plot has to be represented via a list of (x,f(x)) 
      dictOfSetto
   '''
   theplots = {}
   for plotname in dictOfSettings.keys():
      theplots[plotname] = "\\addplot+[%s] coordinates {" % dictOfSettings[plotname]
      lastycoord = -1;
      lastelem = dictOfPlots[plotname][-1]
      for element in dictOfPlots[plotname]:
         xcoord, ycoord = element
         if ycoord != lastycoord or element == lastelem:
            theplots[plotname] += '(%5.1f,%4.1f)' % (xcoord, ycoord)
            lastycoord = ycoord
      
      else:
         theplots[plotname] += "} \\closedcycle ;"
         
   thefinalaxis = "\n\\begin{tikzpicture} \n"
   if usexlogaxis and useylogaxis:
      thefinalaxis += "\\begin{loglogaxis}"
   elif usexlogaxis:
      thefinalaxis += "\\begin{semilogxaxis}"
   elif useylogaxis:
      thefinalaxis += "\\begin{semilogyaxis}"
   else:
      thefinalaxis += "\\begin{axis}"
      
   thefinalaxis += "\n"
   if settings != '':
      thefinalaxis += '[%s]' % settings
   
   for plotname in dictOfPlots.keys():
      thefinalaxis += theplots[plotname] + "\n"
      if legend:
         thefinalaxis += "\\addlegendentry{%s}" % (Misc.getTexName(plotname)) + "\n"
   
   if usexlogaxis and useylogaxis:
      thefinalaxis += "\\end{loglogaxis}"
   elif usexlogaxis:
      thefinalaxis += "\\end{semilogxaxis}"
   elif useylogaxis:
      thefinalaxis += "\\end{semilogyaxis}"
   else:
      thefinalaxis += "\\end{axis}"
         
   thefinalaxis += "\n\\end{tikzpicture} \n"
   
   return thefinalaxis




def getNameOfPicFile(testrunname, probname):
   '''
      defines the name of a single picture file
      the file name is generic in the sense that its file extension is missing, but can be 
      accessed via the format-command of python, e.g., filename.format('.tex')
   '''
   return testrunname + probname + '{0}'

def checkPath(filename):
   ''' test if directory to contain a specified filename exists '''
   pathname = os.path.dirname(filename)
   if pathname == '':
      return True
   if not os.path.exists(pathname):
      print 'Making the directory', pathname
      os.makedirs(pathname)
      print os.path.exists(pathname), ' path existiert'
   return os.path.exists(pathname)
   
def testrungetGapAtTime(testrun, probname, time):
   listofpoints = testrun.problemGetData(probname, PrimalBoundHistoryReader.datakey)
   optimum = float(testrun.problemGetOptimalSolution(changeNameForSoluFile(probname)))
   if listofpoints == []:
      return cutoffGap;
   
   idx = -1
   while idx < len(listofpoints) -1 and float(listofpoints[idx + 1][0]) <= time:
      idx += 1 
   
   if idx == -1:
      assert float(listofpoints[0][0]) > time
      return cutoffGap
   
   assert float(listofpoints[idx][0]) <= time and (idx == len(listofpoints) - 1 or float(listofpoints[idx + 1][0]) > time)
   
   return Misc.getGap(float(listofpoints[idx][1]), optimum , useCplexGap=True)

def execute(comparator, myprobnamelist):
   
   THE_TIMELIMIT = max(comparator.testruns[0].problemlistGetData(myprobnamelist, TimeLimitReader.datakey))
   THE_TIMELIMIT = min(THE_TIMELIMIT, DEFAULT_TIMELIMIT)
   print "time limit for mean integrals set to", THE_TIMELIMIT
   # # mean integral evaluation points
   meanintegralpoints = int(THE_TIMELIMIT)
   scale = THE_TIMELIMIT / float(meanintegralpoints)

   therange = np.arange(meanintegralpoints)

   integralvalues = {}
   meanintegrals = {}
   virtualtestrun = np.zeros(meanintegralpoints)
   
   for testrun in comparator.testruns:
      integralvalues[testrun] = {}
      meanintegrals[testrun] = np.zeros(meanintegralpoints)
   
   nprocessedinstances = 0
   
   
   try:
      checkPath(integralsfilename)
      f = open(integralsfilename, 'w')
   except IOError:
      print 'File', integralsfilename, "doesn't exist!!!"
       
   # ## print head of LaTeX file
   f.write(getTexHeader('a4paper,bindingoffset=0.5cm,twoside,includehead,bottom=2cm') + "\n")

   f.write(getTexAuthor() + "\n")

   f.write(getTexPgfplotsset('every axis') + "\n")



   f.write('''\\begin{document}

   \centering

   ''')
   for probname in myprobnamelist :
      
      optimalsolution = comparator.testruns[0].problemGetOptimalSolution(changeNameForSoluFile(probname))

         
      optimalfloat = float(optimalsolution)
      
      f.write(getTexSection(probname))
      assert(optimalfloat != Misc.FLOAT_INFINITY), probname
      
      timelimithit = False;
      nprocessedinstances += 1
      
      for testrun in comparator.testruns:
         timelimithit = timelimithit or testrun.timeLimitHit(probname)
      
      optimum = comparator.testruns[0].problemGetOptimalSolution(changeNameForSoluFile(probname))
      optimum = float(optimum)
   
      maximumgap = max(1, getMaxGap(probname, comparator.testruns, optimum))
      if not timelimithit:
         maximumtime = getMaxTime(probname, comparator.testruns)
      else:
         maximumtime = comparator.testruns[0].problemGetData(probname, TimeLimitReader.datakey)
      
      thelocalsettings = '''xmax = %f,
                           ymax = %f''' % (maximumtime, maximumgap)
                           
      f.write(getTexPgfplotsset('local settings', thelocalsettings) + "\n")
      
      for testrun in comparator.testruns:
               
         mydatalist = testrun.problemGetData(probname, PrimalBoundHistoryReader.datakey)
         
         if mydatalist == []:
            firstpointintime = testrun.problemGetData(probname, TimeLimitReader.datakey);
            # continue
         else :
            time, bound = mydatalist[0];
            firstpointintime = float(time);
            
         dictOfPlots = {}
         dictOfSettings = {}
         firstplotname = 'firstplot'
         dictOfSettings[firstplotname] = 'fill=orange!40,mark=none,draw=red'
         dictOfPlots[firstplotname] = [(0.0, 1.1 * max(maximumgap, 1)), (firstpointintime, 1.1 * max(maximumgap, 1))]
         secondplotname = 'secondplot'
         dictOfPlots[secondplotname] = []
         dictOfSettings[secondplotname] = 'fill=orange'
         gap = 100.0
         for datatoken in mydatalist:
            time, bound = datatoken
            boundfloat = float(bound);
            gap = min(gap, Misc.getGap(boundfloat, optimalfloat, useCplexGap=True));
               
            dictOfPlots[secondplotname].append((float(time), gap))
               
         else:
            if testrun.timeLimitHit(probname) or gap > 10e-6:
               dictOfPlots[secondplotname].append((testrun.problemGetData(probname, TimeLimitReader.datakey), gap))
               
         solvingtime = float(testrun.problemGetData(probname, SolvingTimeReader.datakey))
         integralvalues[testrun][probname] = calcIntegralValue(mydatalist, optimalfloat, cutoffGap, min(solvingtime, testrun.problemGetData(probname, TimeLimitReader.datakey)))
         settings = '''local settings,
         title={settings: %s, integral value: %f}''' % (Misc.getTexName(testrun.getIdentification()), integralvalues[testrun][probname])
            
         f.write(getAxisPlot(dictOfPlots, dictOfSettings, settings))
      else:
         f.write("\\newpage")
            
   f.write('\\end{document}')



   for probname in myprobnamelist:
      for i in range(meanintegralpoints):
         theminattimeI = cutoffGap
         for testrun in comparator.testruns:
            gap = testrungetGapAtTime(testrun, probname, scale * i)
            
            meanintegrals[testrun][i] += gap 
            if  gap < theminattimeI:
               theminattimeI = gap
         virtualtestrun[i] += theminattimeI
         
      
   virtualtestrun /= float(nprocessedinstances)
   
   for testrun in comparator.testruns:
      meanintegrals[testrun] = meanintegrals[testrun] / float(nprocessedinstances)
      

     
   if writemeanintegrals:
      try:
         checkPath(meanintegralsfilename)
         f = open(meanintegralsfilename, 'w')
      except IOError:
         print 'File doesnt exist'
         
      colors = ['blue', 'red', 'green', 'yellow', 'black', 'gray']
      dictionary = {}
      settingsdictionary = {} 
      for testrun in comparator.testruns:
         thepointlist = []
         for i in therange:
            thepointlist.append((scale * i, meanintegrals[testrun][i]))
         
         ident = testrun.getIdentification()  # .split(':')[-1]
         dictionary[ident] = thepointlist
         settingsdictionary[ident] = '%s!50!black, fill=%s!30, mark=none, opacity=0.5' % (colors[comparator.testruns.index(testrun)%len(colors)], colors[comparator.testruns.index(testrun)%len(colors)])
      
      virtuallist = []
      for i in therange:
         virtuallist.append((scale * i, virtualtestrun[i]))
      dictionary['virtual'] = virtuallist
      settingsdictionary['virtual'] = '%s!50!black, fill=%s!30, mark=none, opacity=0.5' % (colors[-1], colors[-1])
      f.write(getTexHeader())
      # f.write('\pagestyle{empty}\n')
      f.write(getTexPgfplotsset('every axis'))
      f.write('''\\begin{document}''')
      f.write(getAxisPlot(dictionary, settingsdictionary, legend=True))
      
      for testrun in comparator.testruns:
         thepointlist = []
         for i in therange:
            thepointlist.append((scale * i, meanintegrals[testrun][i]))
         
         dictionary = {'theonlyplot' : thepointlist}    
         settingsdictionary = {'theonlyplot' : 'fill=orange, mark=none'}
         integralvalslist = []
         for probname in myprobnamelist:
            integralvalslist.append(integralvalues[testrun][probname])
         
         
            
         print "Mean integral for ", testrun.getIdentification(), ": ", np.sum(meanintegrals[testrun]) * scale
         f.write(getTexSection("Settings: %s, Integral value: %f" % (testrun.getIdentification(), np.sum(meanintegrals[testrun]) * scale)))
         f.write(getAxisPlot(dictionary, settingsdictionary))
      f.write("\\end{document}")   
      f.close()

## execute the main function of the module
if __name__ == "__main__":
   if filelist == []: ## don't continue if no .out file is given for evaluation
      execute = False
      print "Empty file list ---> Cancelling because nothing to do"
     
   if solufilename == '':
      execute = False
      print "Error: No .solu file specified"
   if execute:
      ## initialize the comparator object
      comparator = Comparator(filelist, listofreaders, testsetname, solufilename) 
      
      print "Collecting Data"
      comparator.collectData()

      myprobnamelist = comparator.probnamelist[:] ## copies the list of instance names found during data collection
      deletedproblems = [] ## list to record the deleted instances
      
      for probname in comparator.probnamelist: # search the instance list for problems which should be excluded
         index = myprobnamelist.index(probname)
        
         if probname in excludeProbs or comparator.excludeProb(changeNameForSoluFile(probname), excludereasons):
            print probname, "EXCLUDED"
            myprobnamelist.remove(probname)
            deletedproblems.append(probname)
      
      print "Calculating integrals and storing them to files ", integralsfilename, "and", meanintegralsfilename      
      execute(comparator, myprobnamelist)
      print "Total number of solutions: ", PrimalBoundHistoryReader.totalnumberofsols



#!/usr/bin/env python3
import sys
import re
import os
import tempfile
from gurobipy import *


if len(sys.argv) < 2:
   print('Usage: gurobi.sh checkcuts.py filename [cutnamefilter]')
   quit()

filename = sys.argv[1]
cutclassname = ""
if len(sys.argv) == 3:
   cutclassname = sys.argv[2]

base=os.path.basename(filename)
prefix=os.path.splitext(base)[0]

basemodelname=prefix + "_withoutcuts.lp"
lazyconss = None

with open (filename, "r") as myfile:
   data=myfile.readlines()
   lazyconsstart = data.index("lazy constraints\n")
   boundstart = data.index("Bounds\n")
   lines1 = data[:lazyconsstart]
   lines2 = data[boundstart:]
   lazyconss = data[lazyconsstart+1:boundstart]

   with open(basemodelname, "w") as outfile:
      for line in lines1:
         outfile.write(line)

      for line in lines2:
         outfile.write(line)

model = read(basemodelname)
os.remove(basemodelname)
model.setParam("OutputFlag", 0);
model.setParam("FeasibilityTol", 1e-9)

validcuts = 0
invalidcuts = 0
currcons = ""
for lazycons in lazyconss:
   currcons += lazycons
   m = re.match("([^:]*):([^<]*)(<=|>=)([^\n]*)", currcons)
   if m is None:
      currcons = currcons[:-1]
      continue
   currcons=""
   consname = m.group(1).strip()
   if not cutclassname in consname:
      continue
   cons = m.group(2).strip()
   rhs = float(m.group(4).strip())

   if m.group(3) == "<=":
      objsense = GRB.MAXIMIZE
   else:
      objsense = GRB.MINIMIZE

   objexpr = LinExpr()
   while len(cons) > 0:
      m = re.match("[\+-][0-9][^ ]*[^\+-]*", cons)
      length = len(m.group(0))
      term = m.group(0).strip()
      coef,varname = term.split()
      cons = cons[length:]
      var = model.getVarByName(varname)
      objexpr.add(var, float(coef))

   model.setObjective(objexpr, objsense)
   model.optimize()
   if model.status == GRB.Status.INF_OR_UNBD:
      print("\nError: model is infeasible or unbounded")
      quit()

   viol = model.objVal - rhs

   if viol > 1e-9:
      solname = prefix + "_" + consname + ".sol"
      modelfilename = prefix + "_" + consname + ".lp"
      model.write(solname)
      model.write(modelfilename)
      print("cut " + consname + " is invalid!")
      print("\tmodel to verify cut has been written to " + modelfilename)
      print("\tinteger solution that violates the cut by "+ str(viol) + " was written to " + solname)
      invalidcuts += 1
   else:
      validcuts += 1

print("found "+str(invalidcuts)+" invalid and "+str(validcuts) + " valid cuts")



from math import *
from TestRun import TestRun

class DataCollector:
   '''
      acquires test run data
   '''  
   listofreaders = []
   register_err = 'register error: Reader already in list of readers'
   readernotfound_err = 'Error: Reader is not registered'
   ndata_err = 'Number Error: Readers collected data for a different number of instances'
   filestring = ''
   index = -1
   datacollected = False
   testrun = None

   def __init__(self, testrun = None, index=0):
      self.index = index
      self.listofreaders = []
      self.testrun = testrun
      if self.testrun != None:
         self.filestring = self.testrun.filename

   def setTestRun(self, testrun):
      if self.testrun != testrun:
         self.testrun = testrun
         self.filestring = self.testrun.filename
         for reader in self.listofreaders:
            reader.setTestRun(testrun)
         self.datacollected = False

   def getNReaders(self):
      return len(self.listofreaders)
   
   
   def registerReader(self, reader):
      for handl in self.listofreaders:
         if reader.getName() == handl.getName():
            raise self.register_err
            
      self.listofreaders.append(reader)
      reader.setTestRun(self.testrun)

   def registerListOfReaders(self, readers):
      for reader in readers:
         self.registerReader(reader)


   def collectData(self):
      assert(self.testrun != None)
      f = None
      try:
         f = open(self.filestring, 'r')
      except IOError:
         print 'File', self.filestring, "doesn't exist!!!" 
      
      for reader in self.listofreaders:
         reader.initializeForTestrun()
      
      for line in f:
         for reader in self.listofreaders:
            data = reader.operateOnLine(line)
            if data != None:
               self.testrun.addData(data)
            
      f.close()
      #print("Collection of data finished")
      self.datacollected = True
      self.testrun.setDataCollected(True)
      return 1
      
   def printData(self):
      ninstances = -1;
      for reader in self.listofreaders:
         if ninstances == -1:
            ninstances = reader.getNInstances()
         else:
            if ninstances != reader.getNInstances():
               raise self.ndata_err
            
         
      for reader in self.listofreaders:
         print reader.columnheaderstr,
      else:
         print
      
      for reader in self.listofreaders:
         print reader.columnwidth * '-',
      else:
         print
      
      
      for i in range(0, ninstances):
         for reader in self.listofreaders:
            print reader.getStatisticForInstance(i),
         else:
            print
         
      return 1
      
      



import Misc
from StatisticReader_SolvingTimeReader import SolvingTimeReader
from StatisticReader_TimeLimitReader import TimeLimitReader

class TestRun:
   '''
   represents the collected data of a particular log (.out) file
   '''
   solufilename = ''
   testsetname = ''
   filename = ''
   data = {}
   solufiledata = {}
   problist = []
   nprobs = 0
   settings = 'default'
   version = ''
   lpsolver = ''
   solver = ''
   datacollected = False
   INFINITY = 1e20
   
   solufile_err = 'ERROR: No Solufile known'
   
   def __init__(self, filename = '', solufilename = '', testsetname = '' ):
      if solufilename != '':
         self.solufilename = solufilename 
   
      if testsetname != '':
         self.testsetname = testsetname
   
      if filename != '':
         self.filename = filename 
   
      self.data = {}
      self.solufiledata = {}
      self.problist= []
      self.nprobs = 0
      self.settings = ''
      self.version = ''
      self.lpsolver = ''
   
   def setDataCollected(self, datacollected):
      self.datacollected = datacollected
   
   def addProblem(self, probname):
      assert not (probname in self.problist)
      assert not (probname in self.data.keys())
      #print probname
      self.problist.append(probname)
      self.data[probname] = {}
      self.nprobs = self.nprobs + 1
   
   def __addData(self, probname, datakey, datum):
      assert probname in self.data.keys()
      if datakey in self.data[probname].keys():
         print self.data[probname].keys(), datakey
      
      assert not datakey in self.data[probname].keys()
   
      self.data[probname][datakey]= datum
   
   def addData(self, data):
      assert data != None
      probname, datakey, datum = data
      self.addInternalData(probname, datakey, datum)
      
   def addInternalData(self, probname, datakey, datum):
      if not (probname in self.problist):
         self.addProblem(probname)
      
      self.__addData(probname, datakey, datum)
      
      
   def problemGetName(self, index):
      return self.problist[index]
   
   def problemGetData(self, probname, datakey):
      return self.data[probname][datakey]
       
   def problemlistGetData(self, problemlist, datakey, numeric=True):
      array=[]
      for probname in problemlist:
         data = self.problemGetData(probname, datakey)
         if numeric:
            data = float(data)
         
         array.append(data)
      return array
   
   def getSettings(self):
      return self.settings
   
   def getVersion(self):
      return self.version
   
   def getLpSolver(self):
      return self.lpsolver
   
   def getIdentification(self):
      return self.solver + '('+self.getVersion() + ')' + self.getLpSolver() + ':' + self.getSettings()
   def getShortIdentification(self, char='_', maxlength=-1):
      return Misc.cutString(self.getSettings(), char, maxlength)
   
   def isProblemSolved(self, probname):
      return True
   
   def isProblemFeasible(self, probname):
      return True
   
   def problemGetIndex(self, probname):
      return self.problist.index(probname)
   
   def timeLimitHit(self, probname):
      return self.data[probname][SolvingTimeReader.datakey] - self.data[probname][TimeLimitReader.datakey] >= 0
   
   def getTestset(self):
      self.testsetname
       
   
   
   def problemGetOptimalSolution(self, solufileprobname):
      if self.solufiledata == {}:
         raise self.solufile_err
         
      objval = self.solufiledata[solufileprobname][0]
      
      return objval
       
   def problemGetSoluFileStatus(self, solufileprobname):
      if self.solufiledata == {}:
         raise self.solufile_err
      
      status = self.solufiledata[solufileprobname][1]
      
      return status
      



#! /usr/bin/env python2
# -*- coding: utf-8 -*-
import sys
import commands

'''
This script updates the SCIP version

Invokation via

./updateversion.py [-a] [-v <VERSION>]

where -a signals that an API change was done and the API version
number should be increased (this is the default when providing no arguments)
and -v <VERSION> set the SCIP version number to VERSION, e.g., 4.0.0.

'''

# search for parameters
vpos = -1
apos = -1
for i in range(1,len(sys.argv)):
    argument = sys.argv[i]
    if argument == "-a":
        apos = i
    if argument == "-v":
        vpos = i
if vpos == -1:
    apos = 1

# check correct usage
if len(sys.argv) == 1 or len(sys.argv) > 4 or vpos == len(sys.argv)-1 or vpos == apos -1:
    print("usage: scripts/updateversion.py [-a] [-v <MAJOR>.<MINOR>.<PATCH>]")
else:
    # old version numbers
    oldmajor = newmajor = commands.getoutput("grep 'SCIP_VERSION_MAJOR =' make/make.project").split()[2]
    oldminor = newminor = commands.getoutput("grep 'SCIP_VERSION_MINOR =' make/make.project").split()[2]
    oldpatch = newpatch = commands.getoutput("grep 'SCIP_VERSION_PATCH =' make/make.project").split()[2]
    oldapiversion = newapiversion = commands.getoutput("grep 'SCIP_VERSION_API =' make/make.project").split()[2]

    if apos != -1:
        newapiversion = str(int(oldapiversion)+1)
        print("update SCIP API version from %s to %s" %(oldapiversion, newapiversion))

    if vpos != -1:
        tokens = sys.argv[vpos+1].split(".")
        assert len(tokens) == 3 or len(tokens) == 4, "The version number must consist of 3 or 4 numbers, divided by '.'s (is: "+sys.argv[vpos+1]+")"

        # new version numbers
        newmajor = tokens[0]
        newminor = tokens[1]
        newpatch = tokens[2]

    newversion = newmajor+newminor+newpatch

    oldversionstring = oldmajor + "." + oldminor + "." + oldpatch
    newversionstring = newmajor + "." + newminor + "." + newpatch

    if newversionstring != oldversionstring:
        print("update SCIP version from %s to %s" %(oldversionstring, newversionstring))

    # update version numbers
    commands.getoutput('sed -i "s/\@version.*/\@version  %-s/" doc/xternal.c' %(newversionstring))
    commands.getoutput('''sed -i 's/^VERSION=.*/VERSION=\"%s\"/' scripts/makedist.sh''' %(newversionstring))
    commands.getoutput('sed -i "s/^SCIP_VERSION_MAJOR.*/SCIP_VERSION_MAJOR = %-s/" make/make.project' % newmajor)
    commands.getoutput('sed -i "s/^SCIP_VERSION_MINOR.*/SCIP_VERSION_MINOR = %-s/" make/make.project' % newminor)
    commands.getoutput('sed -i "s/^SCIP_VERSION_PATCH.*/SCIP_VERSION_PATCH = %-s/" make/make.project' % newpatch)
    commands.getoutput('sed -i "s/set(SCIP_VERSION_MAJOR %s)/set(SCIP_VERSION_MAJOR %s)/" CMakeLists.txt' %(oldmajor, newmajor))
    commands.getoutput('sed -i "s/set(SCIP_VERSION_MINOR %s)/set(SCIP_VERSION_MINOR %s)/" CMakeLists.txt' %(oldminor, newminor))
    commands.getoutput('sed -i "s/set(SCIP_VERSION_PATCH %s)/set(SCIP_VERSION_PATCH %s)/" CMakeLists.txt' %(oldpatch, newpatch))

    if newapiversion != oldapiversion:
        print("\nAPI versions before the change:")
        print(commands.getoutput('grep -e "APIVERSION" -e "SCIP_VERSION_API" make/make.project CMakeLists.txt'))
        if newminor != "0" or newpatch != "0":
            print("\nWarning: API version increased for what does not seem to be the master branch (version %s)" %(newversionstring))
        commands.getoutput('sed -i "s/^SCIP_VERSION_API.*/SCIP_VERSION_API = %-s/" make/make.project' % newapiversion)
        commands.getoutput('sed -i "s/set(SCIP_VERSION_API [0-9]*)/set(SCIP_VERSION_API %s)/" CMakeLists.txt' \
                           %(newapiversion))
        print("\nAPI versions after the change:")
        print(commands.getoutput('grep -e "APIVERSION" -e "SCIP_VERSION_API" make/make.project CMakeLists.txt'))



from StatisticReader import StatisticReader
import re

class PrimalBoundHistoryReader(StatisticReader):
   name = 'PrimalBoundHistoryReader'
   regular_exp =  ' time | node  | left  |LP iter|LP it/n| mem |mdpt |frac |vars |cons |cols |rows |cuts |confs|strbr|  dualbound   | primalbound  |  gap   '
   columnwidth = 20
   datakey = 'PrimalBoundHistory'
   columnheaderstr = 'PrimalBoundHistory'.rjust(columnwidth)
   inTable = False
   lastPrimalBound = '--'
   listOfPoints = []
   easyCPLEX= True
   totalnumberofsols = 0
       
   testnumberofsols = 0
   gurobiextralist = []

   def extractStatistic(self, line):
      if StatisticReader.solvertype == StatisticReader.SOLVERTYPE_SCIP:
         if self.regular_exp in line:
            self.inTable = True
         elif self.inTable and re.search("s\|", line):
   #         print line
            mylist = line.split()
   #         print mylist
            if len(mylist) == 0:
               return None
            if re.search('s\|', mylist[0]):
               rawtime = mylist[0].split('s|')[0]
            else:
               assert(re.search('s|', mylist[1]))
               rawtime = mylist[1].split('s|')[0]
                     
            rawtime = re.split('[a-zA-Z*]', rawtime)
                  
            pointInTime = rawtime[len(rawtime) - 1]
            if re.search("\|",mylist[len(mylist) - 1]):
               PrimalBound = mylist[len(mylist) - 2].strip("|")
            else:
               PrimalBound = mylist[len(mylist) - 3].strip("|")
   #         print pointInTime, PrimalBound
            if PrimalBound != self.lastPrimalBound:
               self.lastPrimalBound = PrimalBound
               try:
                   self.listOfPoints.append((float(pointInTime),float(PrimalBound)))
               except ValueError:
                   pass

         elif "SCIP Status" in line and self.inTable:
            self.inTable = False

      elif StatisticReader.solvertype == StatisticReader.SOLVERTYPE_GUROBI:
         if "Found heuristic solution" in line:
            self.gurobiextralist.append(line.split()[-1])
         if "Expl Unexpl |  Obj  Depth" in line:
            self.inTable = True
         elif self.inTable and line.endswith("s\n") and self.gurobiextralist != []:
#            print "+++++++++++++++++++++"
            pointInTime = line.split()[-1].strip("s")
            self.listOfPoints.append((float(pointInTime), float(self.gurobiextralist[-1])))
            self.gurobiextralist = []
         elif self.inTable and line.startswith("H") or line.startswith("*"):
            self.readBoundAndTime(line, -5, -1, timestripchars="s")
             
         elif "Cutting planes:" in line and self.inTable:
            self.inTable = False
         elif self.gurobiextralist != [] and "Explored " in line:
#            print "-------------------------"
            pointInTime = line.split()[-2]
            self.listOfPoints.append((float(pointInTime), float(self.gurobiextralist[-1])))
            self.gurobiextralist = []
          
         return None
            
      elif StatisticReader.solvertype == StatisticReader.SOLVERTYPE_CBC:
         if "Integer solution of " in line:
            self.readBoundAndTime(line, 4, -2, timestripchars="(")
             
         return None
          
      elif StatisticReader.solvertype == StatisticReader.SOLVERTYPE_XPRESS:
         if "BestSoln" in line:
            self.xpresscutidx = line.index("BestSoln") + len("BestSoln")
         elif line.startswith("+") or line.startswith("*"):
            self.readBoundAndTime(line, -1, -1, cutidx=self.xpresscutidx)
         elif line.startswith(" *** Heuristic solution found: "):
            self.readBoundAndTime(line, -4, -2)
             
      elif StatisticReader.solvertype == StatisticReader.SOLVERTYPE_CPLEX:
         if "Solution pool: " in line:
            self.testnumberofsols = int(line.split()[2])
            # assert len(self.listOfPoints) >= self.testnumberofsols
         if self.easyCPLEX and "Found incumbent of value" in line:
            splitline = line.split()
            self.readBoundAndTime(line, splitline.index("Found") + 4, splitline.index("Found") + 6)
         elif not self.easyCPLEX:
            if "Welcome to IBM(R) ILOG(R) CPLEX(R)" in line:
               self.lastelapsedtime = 0.0
               self.nnodessincelastelapsedtime = 0
               self.lastnnodes = 0
               self.cpxprimals = []
                
            if "   Node  Left     Objective  IInf  Best Integer    Best Bound    ItCnt     Gap" in line:
               self.inTable = True

               self.cpxbestintegeridx = line.index("Best Integer") + 11
                
            elif self.inTable and ("cuts applied:" in line or "Root node processing" in line):
               self.inTable = False
            elif self.inTable and "Repeating presolve." in line:
               self.inTable = False
            elif self.inTable and "Covers:" in line:
               self.inTable = False
            elif self.inTable and len(line) > 0 and line.startswith(" ") or line.startswith("*"):
               if line=="\n":
                  return None
               nodeinlineidx = 7
               while line[nodeinlineidx] != " " and line[nodeinlineidx] != "+":
                  nodeinlineidx += 1
               nnodes = int(line[:nodeinlineidx].split()[-1].strip('*+')) + 1
               if line.startswith("*") or line.startswith("+"):
                  print line
                  primalbound = float(line.split()[-4])
                  print primalbound, nnodes
                  self.cpxprimals.append((nnodes, primalbound))
               self.lastnnodes = nnodes
            elif "Elapsed time = " in line:
               thetime = float(line.split()[3])
               self.processCpxprimals(thetime)
                
               self.nnodessincelastelapsedtime = self.lastnnodes
               self.lastelapsedtime = thetime

            elif "Solution time =" in line:
               thetime = float(line.split()[3])
               self.processCpxprimals(thetime)
       
      return None
       
   def readBoundAndTime(self, line, boundidx, timeidx, timestripchars="", cutidx=-1):
      splitline = line.split()
        
      primalbound = line[:cutidx].split()[boundidx]
        
      pointInTime = splitline[timeidx].strip(timestripchars)
             
#        print line 
#        print pointInTime, primalbound
        
        
      self.lastPrimalBound = primalbound
      self.listOfPoints.append((float(pointInTime), float(primalbound)))
       
   def processCpxprimals(self, currenttime):
      solvednodes = (self.lastnnodes - self.nnodessincelastelapsedtime)
      timespentonnode = (currenttime - self.lastelapsedtime)/max(1.0, float(solvednodes))
      assert currenttime >= self.lastelapsedtime
      for node, bound in self.cpxprimals:
         estimatedtime = self.lastelapsedtime + (node - self.nnodessincelastelapsedtime) * timespentonnode
                   
         if bound != self.lastPrimalBound:
            self.lastPrimalBound = bound
            if len(self.listOfPoints) > 0:
               othertime, otherbound = self.listOfPoints[-1]
               assert othertime <= estimatedtime
               self.listOfPoints.append((float(estimatedtime), float(bound)))
               
      self.cpxprimals = []

   def execEndOfProb(self):
      self.inTable = False
      self.testnumberofsols = 0
      theList = self.listOfPoints[:]
      self.listOfPoints = []
      self.lastPrimalBound = '--'
      PrimalBoundHistoryReader.totalnumberofsols += len(theList)
      #print " solutions added:", PrimalBoundHistoryReader.totalnumberofsols
      return theList


from StatisticReader import StatisticReader

class SoluFileReader(StatisticReader):
   '''
   reader for solu file data
   '''
   name = 'SoluFileReader'
   regular_exp = ['=opt=', '=inf=', '=unkn=', '=best=', '=feas=']
   actions = {}
   datakey = 'SoluFileInformation'
   statistics = {}
   conflict_err = ' Error: instance already stored in solution statistics'
   columnwidth = 12
   columnheaderstr = 'SoluFile'.rjust(columnwidth)

   INFINITY = 1e20

   def __init__(self):
      self.__initActions()

   def setTestRun(self, testrun):
      self.testrun = testrun
      if testrun != None:
         self.statistics = self.testrun.solufiledata

   def extractStatistic(self, line):
      assert self.testrun != None
      for regexp in self.regular_exp:
         if regexp in line:
            newaction = self.actions[regexp]
            newaction(line)
      else:
         return None
            

   def __storeToStatistics(self, instance, objval, status):
      if not instance in self.statistics.keys():
         self.statistics[instance] = (objval, status)
      else:
         raise self.conflict_err


   def __newOptInstance(self, line):
      splittedline = line.split()
      assert splittedline[0] == '=opt='
      instance = splittedline[1]
      objval = splittedline[2]

      self.__storeToStatistics(instance, objval, status='opt')

   def __newInfInstance(self, line):
      splittedline = line.split()
      assert splittedline[0] == '=inf='
      instance = splittedline[1]
      objval = self.INFINITY

      self.__storeToStatistics(instance,  objval, status='inf')

   def __newUnknInstance(self, line):
      splittedline = line.split()
      assert splittedline[0] == '=unkn='
      instance = splittedline[1]
      objval = self.INFINITY

      self.__storeToStatistics(instance, objval, status='unkn')


   def __newBestInstance(self, line):
      splittedline = line.split()
      assert splittedline[0] == '=best='
      instance = splittedline[1]
      objval = splittedline[2]

      self.__storeToStatistics(instance, objval, status='best')

   def __newFeasInstance(self, line):
      splittedline = line.split()
      assert splittedline[0] == '=best='
      instance = splittedline[1]
      objval = self.INFINITY
        
      self.__storeToStatistics(instance, objval, status='feas')



   def __initActions(self):
      self.actions = {}
      self.actions['=opt='] = self.__newOptInstance
      self.actions['=inf='] = self.__newInfInstance
      self.actions['=unkn='] = self.__newUnknInstance
      self.actions['=best='] = self.__newBestInstance
      self.actions['=feas='] = self.__newFeasInstance
        



#! /usr/bin/env python

import clang.cindex
import sys
import clang
import os.path
from clang.cindex import CursorKind, TokenKind, TypeKind, StorageClass

clang.cindex.Config.set_library_file("/usr/lib/llvm-5.0/lib/libclang.so.1")
includedirs = ['../../src/','newfiles/']

def remove_prefix(p,s):
    if s.startswith(p):
        return s[len(p):]
    return s

def get_headerset(file):
    idx = clang.cindex.Index.create()
    tu = idx.parse(file, args= map(lambda x: '-I' + x, includedirs) + ['-DSCIP_WITH_ZLIB'])

    headers = set()
    for idx, c in enumerate(tu.cursor.walk_preorder()):
        if str(c.location.file) != file:
            continue

        referenced = None

        if c.kind == CursorKind.PARM_DECL:
            thetype = c.type
            while thetype == TypeKind.POINTER:
                thetype = thetype.get_pointee()
            if thetype.kind == TypeKind.TYPEDEF:
                referenced = thetype.get_declaration()
        elif c.kind == CursorKind.FUNCTION_DECL:
            thetype = c.result_type
            while thetype == TypeKind.POINTER:
                thetype = thetype.get_pointee()
            if thetype.kind == TypeKind.TYPEDEF:
                referenced = thetype.get_declaration()
        elif c.kind == CursorKind.CALL_EXPR or c.kind == CursorKind.TYPE_REF or CursorKind.MEMBER_REF_EXPR or CursorKind.DECL_REF_EXPR:
            referenced = c.referenced

        if referenced is not None:
            #print("\n".join(dir(c)))
            #sys.exit(0)
            headerfile = str(referenced.location.file)

            if not headerfile.endswith("c"):
                for p in includedirs:
                    headerfile = remove_prefix(p, headerfile)

                headers.add(headerfile)
    return headers

if __name__ == "__main__":
    inclset = get_headerset(sys.argv[1])
    includelist = []
    if sys.argv[1].endswith(".c"):
        headerfile = sys.argv[1][:-1] + 'h'
        if os.path.isfile(headerfile):
            headerinclset = get_headerset(headerfile)
        else:
            headerinclset = set()

        for p in includedirs:
            headerfile = remove_prefix(p, headerfile)

        for f in headerinclset:
            if f in inclset:
                inclset.remove(f)

        inclset.add(headerfile)
    else:
        includelist.append('#include "scip/def.h"')

    for h in inclset:
        # for system headers only add them if they are not included by scip/def.h and also handle
        # that math functions are defined in mathcalls.h on our linux systems
        if h.startswith("/usr/include/"):
            if h.endswith("mathcalls.h"):
                h = "math.h"
            else:
                h = remove_prefix("/usr/include/", h)
            if h not in ["stdio.h", "stdint.h", "math.h", "limits.h", "float.h", "assert.h"]:
                includelist.append("#include <{}>".format(remove_prefix("/usr/include/", h)))
        else:
            includelist.append('#include "{}"'.format(h))

    print "\n".join(sorted(includelist))



#! /usr/bin/env python

import clang.cindex
import sys
import clang
from clang.cindex import CursorKind, TokenKind, StorageClass

clang.cindex.Config.set_library_file("/usr/lib/llvm-5.0/lib/libclang.so.1")

gap_file_name = "gaps_file.txt"

import argparse

parser = argparse.ArgumentParser(description='Parse input module and header file to create a list of relevant sections')

parser.add_argument('--write_gaps', action='store_true', default=False, dest="write_gaps")
parser.add_argument('files', nargs=2, help='c file and header file')


args = parser.parse_args()

def fully_qualified(c):
    if c is None:
        return ''
    elif c.kind == CursorKind.TRANSLATION_UNIT:
        return ''
    else:
        res = fully_qualified(c.semantic_parent)
        if res != '':
            return res + '::' + c.spelling
    return c.spelling

def recurse_into_static_functions(cursor, static_functions_dict):
    for d in cursor.walk_preorder():
        if d.kind == CursorKind.CALL_EXPR and d.referenced.storage_class == StorageClass.STATIC:
                                #print "\t->{}".format(fully_qualified(d.referenced))
            if d.referenced.extent.start.line not in static_functions_dict:
                static_functions_dict[d.referenced.extent.start.line] = d.referenced
                recurse_into_static_functions(d.referenced, static_functions_dict)


if __name__ == "__main__":

    header_functions_idx = clang.cindex.Index.create()
    tu  = header_functions_idx.parse(args.files[1], args=['-I../../src'])
    header_functions = set()
    for c in tu.cursor.walk_preorder():
        if c.kind == CursorKind.FUNCTION_DECL and str(c.location.file) == args.files[1]:
            header_functions.add(c.spelling)

    idx = clang.cindex.Index.create()
    tu = idx.parse(args.files[0], args=['-I../../src'])

    function_dict = {}
    kindtypes = dict()
    static_functions_dict = dict()
    for idx, c in enumerate(tu.cursor.walk_preorder()):

        kindtypes[c.kind] = kindtypes.get(c.kind, 0) + 1


        if c.kind == CursorKind.FUNCTION_DECL and c.get_definition() is not None:
            #print("\n".join(dir(c)))
            #sys.exit(0)
            if str(c.location.file).endswith("c"):

                if c.storage_class == StorageClass.STATIC:
                    #print "{} : {}--{}".format(fully_qualified(c.referenced), c.extent.start.line,
                    #
                    #c.extent.end.line)
                    pass


                if c.referenced.spelling in header_functions:
                    #print "{} : {}--{}".format(fully_qualified(c.referenced), c.extent.start.line, c.extent.end.line)

                    function_dict[c.extent.start.line] = c

                    recurse_into_static_functions(c, static_functions_dict)



    #print "\n".join(map(str, kindtypes.items()))
    #
    # loop over documentation that and keep merge tokens that are doxygen comments directly before
    # the start of a new function declaration
    #
    lines = []
    for c in tu.cursor.get_tokens():

        # stop at every comment token
        if c.kind == TokenKind.COMMENT:
            nextlinenumber = c.extent.end.line + 1

            #
            # is this function part of the current header module?
            #
            if  nextlinenumber in function_dict or nextlinenumber in static_functions_dict:

                # uncomment to print the documentation
                #print c.spelling
                thedict = function_dict if nextlinenumber in function_dict else static_functions_dict
                function = thedict[nextlinenumber]
                functionname = fully_qualified(function.referenced)

                #
                # print the whole extent from the beginning to the end.
                #
                #output(functionname, c.extent.start.line, function.extent.end.line)
                lines.append((c.extent.start.line, function.extent.end.line))

    if args.write_gaps:
        with open(gap_file_name, "w") as gap_file:
            start = 1
            i = 0
            gaplist = []
            while i < len(lines) - 1:
                _ , start = lines[i]
                end , _ = lines[i+1]
                if start + 1 <= end - 1:
                    gaplist.append((start + 1, end - 1))
                i += 1

            for (start, end) in gaplist:
                if start <= end:
                    gap_file.write("{} {}\n".format(start, end))
    elif len(lines) > 0:

        #
        # try to open a possibly created gap file
        #
        with open(gap_file_name, "r") as gap_file:
            gaps = {}
            for  line in gap_file:
                gaps_line = map(int, line.split())
                gaps[gaps_line[0]] = gaps_line[1]
            start,end = lines[0]
            end+= 1
            merged_lines = []
            i = 0
            while i < len(lines) - 1:
                nextstart,nextend = lines[i+1]
                nextend += 1
                if end in gaps and gaps[end] == nextstart - 1:
                    end = nextend
                else:
                    merged_lines.append((start,end))
                    start,end = (nextstart,nextend)
                i += 1
            merged_lines.append((start,end - 1))
            lines = merged_lines

        for (start,end) in lines:
            print "{},{}p".format(start,end)



#! /usr/bin/python
# -*- coding: utf-8 -*-
import re
import sys

'''
This script checks for correct synchronization between a
header file and its corresponding source file.

Invokation via

./check_scip_header_files.py [HEADER_FILE.h ...]

where HEADER_FILE.h is the (relative or absolute)
path to a header file, e.g., ../src/scip/scip.h.
Optionally, more than one header file may be passed.

'''

docstartstring = "/\*\*"
docendstring = "\*/"
docstartexpr = re.compile(docstartstring)
docendexpr = re.compile(docendstring)
docexpr = re.compile("/\*\*(.*|(?<!\*/)\n)*\*/", re.MULTILINE)
funcexpr = re.compile("(extern|EXTERN)+\n+(([\S_]+)\s*([a-zA-Z0-9]+)\(\n(.*|(?<!;)\n)*\));",re.MULTILINE) #", re.MULTILINE)

# loop over every header file
for filename in sys.argv[1:]:
    if not filename.endswith(".h"):
        print "  WARNING:", filename, " is not a header file (.h!!!), skipping it"
        continue
    print
    print "  --checking ", filename

    if filename.find("misc_") >= 0:
        print "    --> skipped"
        continue

    print
    print

    # first round: check for comments
    with open(filename, 'r') as currentfile:

        for match in docexpr.finditer(currentfile.read()):
            cfilename = filename.replace("pub_","").replace(".h", ".c")
            with open(cfilename, 'r') as cfile:

                if not match.group(1) in cfile.read():
                    print "WARNING: The documentation string"
                    print match.group(1)
                    print "was not found in %s"%cfilename
                    print

    #second round: check for function signatures
    with open(filename, 'r') as currentfile:
        for match in funcexpr.finditer(currentfile.read()):
            cfilename = filename.replace("pub_","").replace(".h", ".c")
            with open(cfilename, 'r') as cfile:
                if not match.group(2) in cfile.read():
                    print "WARNING: The function signature "
                    print match.group(2)
                    print "was not found in %s"%cfilename
                    print

    print "  --finished", filename

  #indoc = False
  #lines = []
  #for line in currentfile:
    #startfound = docstartexpr.search(line)
    #if startfound:
      #indoc = True
    #if indoc:
      #lines.append(line)

    #endfound = docendexpr.search(line)
    #if endfound and indoc:
      #for theline in lines:
	#print theline
      #indoc = False
      #lines = []
